// funciones.cpp: Cdigo de las funciones de esta DLL
// ------------------------------------------------------------------

#include <stdafx.h>

#include "funciones.h"
#include <math.h>

// ------------------------------------------------------------------
// Funciones exportadas
// Estn reflejadas en el archivo .DEF del proyecto en el mismo orden
// ------------------------------------------------------------------

// Deteccin de blobs de una imagen

int Ejecutar(int iFunc, int iNumCtr, int iNumEnt, int iNumSal, CString& strDirTemp, 
				CString* pStrDatos, CString* pStrImgEntr, CString* pStrImgSal)
{
	int i,j; // Para bucles

	// iFunc: debe ser 0
	// iNumCtr: debe ser 3
	// iNumEntr: debe ser 2
	// iNumSal: debe ser 3
	// strDirTemp: directorio temporal
	// pStrDatos[0]: maximo numero de blobs a devolver
	// pStrDatos[1]: minimo numero de pixels en un blob
	// pStrDatos[2]: enerar momento de Hu (1) o no (0)
	// pStrImgEntr[0]: imagen monocromo a procesar
	// pStrImgSal[0]: imagen float de N filas y 3 columnas con las coordenadas
	//				  y angulos de los N blobs detectados:
	//				    X1 Y1 A1   X2 Y2 A2   ...   XN YN AN 
	// pStrImgSal[1]: imagen monocromo etiquetada
	// pStrImgSal[2]: imagen float de N filas y 7 columnas con los momentos 
	//				  de Hu para los N blobs
	//	                M11-M17 M21-M27 ... MN1-MN7

	// Para depuracion
	#ifdef _DEBUG
		TRACE0("MODDLL: Modulo en ejecucion.\n");
		TRACE("Funcion: %d\nDatos: %d\nEntradas: %d\nSalidas: %d\n", 
			iFunc, iNumCtr, iNumEnt, iNumSal);
		TRACE("Dir. Temporal: %s\n", strDirTemp);
		TRACE0("Datos: ");
		for (i=0; i<iNumCtr; i++)
			TRACE("%d:%s\n", i, pStrDatos[i]);
		TRACE0("Entradas: ");
		for (i=0; i<iNumEnt; i++)
			TRACE("%d:%s\n", i, pStrImgEntr[i]);
		TRACE0("\nSalidas: ");
		for (i=0; i<iNumSal; i++)
			TRACE("%d:%s\n", i, pStrImgSal[i]);
		TRACE0("\n");
	#endif

	// Requiere una imagen de entrada, 3 parmetros y 3 salidas:
	if ((iNumEnt!=1) || (iNumCtr!=3) || (iNumSal!=3))
		return 1;

	// Lee valores de parametros de entrada
	int iMinPixels = atoi((LPCSTR)pStrDatos[0]);
	int iMaxBlobs = atoi((LPCSTR)pStrDatos[1]);
	BOOL bGenMomHu = (atoi((LPCSTR)pStrDatos[2]))>0? TRUE:FALSE;

	// El numero de blobs debe ser entre 1 y 255 (para etiquetar con bytes)
	if ((iMaxBlobs<1) || (iMaxBlobs>255))
		return 2;

	// Crea y lee imagen de entrada
	StCabecera cabEnt;
	BYTE* pImgEntr = LeeImagenEntrada(strDirTemp + pStrImgEntr[0], &cabEnt);
	if (pImgEntr == NULL)
		return 3;

	// Crea vector para coordenadas de los blobs
	StBlob* pBlobs = new StBlob [iMaxBlobs];
	if (pBlobs == NULL)
	{
		delete pImgEntr;
		return 4;
	}

	// Crea imagen de entrada extendida con bordes
	BYTE** pEntrada = CreaImagenXY(cabEnt.wX+1, cabEnt.wY+1);
	if (pEntrada == NULL)
	{
		delete pImgEntr;
		delete pBlobs;
		return 5;
	}

	// Inicia imagen de entrada extendida: Rellena borde y llena a 0 y 1
	WORD wX,wY;
	for(wX=0; wX<=cabEnt.wX; wX++)
		pEntrada[0][wX]=0;
	
	for(wY=0; wY<=cabEnt.wY; wY++)
		pEntrada[wY][0]=0;

	DWORD dwIndData=0;
	for(wY=1; wY<=cabEnt.wY; wY++)
		for(wX=1; wX<=cabEnt.wX; wX++)
            pEntrada[wY][wX]=(pImgEntr[dwIndData++]>127)?1:0;

	// Ya no hace falta la imagen de entrada
	delete pImgEntr;

	// Crea imagen de salida 
	BYTE** pSalida = CreaImagenXY(cabEnt.wX, cabEnt.wY);
	if (pSalida  == NULL)
	{
		delete pBlobs;
		BorraImagenXY(cabEnt.wX+1, cabEnt.wY+1, pEntrada);
		return 6;
	}

	// Determina blobs
	int iNumBlobs=0;
	iNumBlobs = EtiquetaBlobs(cabEnt.wX, cabEnt.wY, pEntrada, pSalida);

	// Elimina blobs pequeos
	iNumBlobs = EliminaBlobs(cabEnt.wX, cabEnt.wY, iNumBlobs, pSalida, 
		iMinPixels);

	// Determina coordenadas de los blobs
	int iCX, iCY;
	for(i=0; i<iNumBlobs; i++)
	{
		CentroGravedadXY(cabEnt.wX, cabEnt.wY, pSalida, i+1, iCX, iCY);
		pBlobs[i].fX = (float)iCX;
		pBlobs[i].fY = (float)iCY;
		pBlobs[i].fA = AnguloEje(cabEnt.wX, cabEnt.wY, pSalida, i+1, iCX, iCY);
	}

	// Cdigo de error a devolver
	int iErr=0;

	// Escribir imagen con coordenadas de blobs
	if (EscribeBlobs(strDirTemp + pStrImgSal[0], iNumBlobs, pBlobs))
	{
		// Escribir imagen etiquetada se salida
		if (EscribeSalida(strDirTemp + pStrImgSal[1], &cabEnt, pSalida))
		{
			// Si hay que calcular momentos de Hu
			if (bGenMomHu)
			{
				// Memoria para los momentos
				float* pMomentos[7];
				for (i=0; (i<7)&&(iErr==0); i++)
				{
					pMomentos[i] = new float[iNumBlobs];
					if (pMomentos[i] == NULL)
						iErr = 7;
				}

				// Si no se reservo toda la memoria, hay que liberar la que se reservo
				if (iErr!=0)
				{
					i--;
					while (i>0)
						delete pMomentos[i--];
				}

				// Calcular y escribir los momentos de Hu
				else
				{
					// Calcular para cada blob y para cada orden
					for (i=0; i<iNumBlobs; i++)
						for (j=0; j<7; j++)
							pMomentos[j][i]=MomentoHu(cabEnt.wX, cabEnt.wY,
								i+1, j+1, pSalida, pBlobs);

					// Guardar
					if (!EscribirMomentosHu(strDirTemp + pStrImgSal[2], 
							iNumBlobs, pMomentos))
						iErr = 8;

					// Liberar memoria
					for (i=0; i<7; i++)
						delete pMomentos[i];
				}
			}
			// Aunque no se generen los momentos de HU, hay que escribir algo
			else
			{
				if (!EscribirMomentosHu(strDirTemp + pStrImgSal[2], 0, NULL))
					iErr = 8;
			}

		}
		else iErr = 9;
	}
	else iErr = 10;

	// Libera memoria 
	BorraImagenXY(cabEnt.wX+1, cabEnt.wY+1, pEntrada);
	BorraImagenXY(cabEnt.wX, cabEnt.wY, pSalida);
	delete pBlobs;	 // Coordenadas de salida 

	
	return iErr;
}

// ------------------------------------------------------------------
// Funciones internas
// ------------------------------------------------------------------

// Leer imagen de entrada
BYTE* LeeImagenEntrada(CString& strNombre, StCabecera* pCab)
{
	CFile file;
	BYTE* pDatos;
	
	// Abrir la imagen de entrada
	if(!file.Open(strNombre, CFile::modeRead | CFile::shareDenyWrite))
		return NULL;

	// Leer cabecera de la imagen de entrada
	if (file.Read(pCab, sizeof(StCabecera)) < sizeof(StCabecera))
	{
		file.Close();
		return NULL;
	}

	// Debe tener almenos tamao 1 de ancho y de alto, y 1 canal
	if ((pCab->wX < 1) || (pCab->wY < 1) || (pCab->wC < 1) || (pCab->wB != 1))
	{
		file.Close();
		return NULL;
	}

	// Calcula tamao de la imagen y reserva memoria
	DWORD dwTam = (DWORD)(pCab->wX) * (DWORD)pCab->wY * (DWORD)pCab->wC;
	pDatos = new BYTE[dwTam];
	if (pDatos==NULL)
	{
		file.Close();
		return NULL;
	}
	
	// Leer datos de la imagen de entrada
	if (file.Read(pDatos, dwTam) < dwTam)
	{
		file.Close();
		return NULL;
	}
	
	// Cerrar archivo de entrada
	file.Close();

	return pDatos;
}

// Crear una imagen de BYTEs en forma de array del tamao indicado
BYTE** CreaImagenXY(WORD wTamX, WORD wTamY)
{
	// Vector de filas
	BYTE** pImg = new BYTE* [wTamY];
	
	if(pImg!=NULL)
	{
		// Para cada fila, un vector de columnas
		for(WORD i=0; i<wTamY; i++)
		{
			pImg[i] = new BYTE [wTamX];

			// Si no puede crear fila, lo borra todo y acaba
			if (pImg[i] == NULL)
			{
				for(WORD j=0; j<i; j++)
					delete pImg[j];

				delete pImg;

				return NULL;
			}
		}
	}

	return pImg;
}

// Borrar una imagen de BYTEs en forma de array del tamao indicado 
void BorraImagenXY(WORD wTamX, WORD wTamY, BYTE** pImg)
{
	// Borra cada uno de los vectores de columnas
	for(WORD i=0; i<wTamY; i++)
		delete pImg[i];

	// Borra el vector de filas
	delete pImg;
}

//Permite la renumeracion de los blobs en el caso que sea necesario
void Reordenar(WORD pos_i, WORD pos_j, BYTE **MA, int num_abajo, int num_arriba,
			   WORD wTamX, WORD wTamY)
{
	WORD i,j;
	BOOL continua=TRUE;

	// Si numblob abajo es mayor a numblob arriba
	if(num_abajo > num_arriba)
	{	
		// El valor de la matriz[pos_i][pos_j]=numblob arriba
		MA[pos_i][pos_j]=num_arriba;

		// Buscamos para reordenar hasta donde llevemos introducida de matriz de imagen
		for(i=0; (i<wTamY) && continua; i++)
			for(j=0; (j<wTamX) && continua;j++)
			{
				// Debemos saber hasta donde debemos comprobar 
				if(i==pos_i && j==pos_j)
					// Indica hemos llegado al fin de matriz de salida
					continua=FALSE;

				// Si no he llegado al final de matriz de copia entonces
    			else
				{
					// Si un pixel es de numblob abajo lo cambio al numblob arriba 
					if(MA[i][j] == num_abajo)
						MA[i][j]=num_arriba;

					// Si un pixel es mayor a numblob abajo le resto 1 para renumerar
					// sin saltos
    				else if(MA[i][j] > num_abajo)
						MA[i][j]=MA[i][j]-1;
				}
			}//for
	}//if

    // Si numblob arriba es mayor a numblob abajo
	else if(num_arriba > num_abajo)
	{
		MA[pos_i][pos_j]=num_abajo;			
		
		//buscamos para reordenar hasta donde llevemos introducida de matriz de imagen
		for(i=0; (i<wTamY) && continua; i++)
			for(j=0; (j<wTamX) && continua; j++)
			{
				// Debemos saber hasta donde debemos comprobar 
    			if(i==pos_i && j==pos_j)
					// Indica hemos llegado al fin de matriz de salida
					continua=FALSE;

				//Si no he llegado al fin de matriz de copia
				else
				{
					// Si un pixel es de num_blob arriba lo cambio al num_blob abajo
					if(MA[i][j]==num_arriba)
						MA[i][j]=num_abajo;

					//Si un pixel es mayor a num_blob arriba le resto 1 para 
					// renumerar sin saltos
					else if(MA[i][j]>num_arriba)
						MA[i][j]=MA[i][j]-1;
				}
   			}//for
   	}//else
}

// Etiqueta la imagen de salida con los blobs, devolviendo el 
// numero de blobs hallados
int EtiquetaBlobs(WORD wTamX, WORD wTamY, BYTE** pEntrada, BYTE** pSalida)
{
	int opcion;
	WORD i,j;
	
	// I ndica el numero de blob por el que voy.
	// Al finalizar indicara el numero total de blobs
	int num_blob = 0;

	// Para cada pixel de la entrada
	// (la matriz de salida la referenciamos con un indice menos)
	for(i=1; i<=wTamY; i++)
		for(j=1; j<=wTamX; j++)
		{
			// Se trata de la mascara y me permite saber a que 
			// caso pertenece cada iteracion
			opcion = (int)pEntrada[i][j];
			opcion += (int)(pEntrada[i-1][j]) * 2;
			opcion += (int)(pEntrada[i-1][j-1])* 4;
			opcion += (int)(pEntrada[i][j-1]) * 8;
		
			switch(opcion)
			{
				// Conectar a blob de fondo
				case 0:
				case 2:
				case 4: // Fin blob objeto conectar a blob de fondo-> sin problema pues solo 1 blob de fondo
				case 6:
				case 8:			
				case 10:			
				case 12: pSalida[i-1][j-1]=0;
						 break;

				case 14: pSalida[i-1][j-1]=0;
					     break;
					
				// Comenzar nuevo blob
				case 1:  num_blob++; 
						 pSalida[i-1][j-1] = num_blob;
						 break;

				// Hay 8-conexion Conectar a blob de objeto
				case 5: pSalida[i-2][j-2] = num_blob;
						break;
												
				// Conectar a blob objeto			
				case 3:
				case 7: pSalida[i-1][j-1] = pSalida[i-2][j-1];
					    break;

				// Conectar a blob objeto
				case 9:			
				case 13:			
				case 15: pSalida[i-1][j-1] = pSalida[i-1][j-2];
						 break;
						
				// Caso particular. Se unen 2 franjas
				case 11:
				{
					// Caso de que las 2 franjas son de distinto blob
					if(pSalida[i-1][j-2] != pSalida[i-2][j-1])
					{
						// i-1 y j-1 indican hasta donde se ha hecho pSalida
						// pSalida[i-1][j-2] indica el numblob del Pixel inferior
						// pSalida[i-2][j-1] indica el numblob del Pixel superior
						Reordenar(i-1, j-1, pSalida, pSalida[i-1][j-2],
							pSalida[i-2][j-1], wTamX, wTamY);
						
						// He restado a todos un blob tengo asi un blob menos
						num_blob--;
					}
								
					// Caso de que las 2 franjas son del mismo blob
					// pSalida[i-1][j-2] == pSalida[i-2][j-1]
					else
					{
						//Da lo mismo igualar al de arriba o al de abajo
						pSalida[i-1][j-1] = pSalida[i-1][j-2];
					}
				}
				break;
			
			} // del switch
		} // del for j

	return num_blob;
}

// Elimina blobs con menos de iMinPixels, y devuelve nuevo numero de blobs
int EliminaBlobs(WORD wTamX, WORD wTamY, int iNumBlobs, BYTE** pSalida, int iMinPixels)
{
	WORD y,x,xr,yr;
	BYTE b, bMax;
	int cuenta;
	
	// Numero maximo de blobs en byte
	bMax = (BYTE)iNumBlobs;

	// Para cada indice de blob: el valor b=0 es para identificar lo que no es blob
	for(b=1; b<=bMax; b++) 
	{
		cuenta=0;
		for(y=0; y<wTamY; y++)
			for(x=0; x<wTamX; x++)
				if(pSalida[y][x]==b)
					cuenta++;

		// Si hay que eliminar el blob
		if (cuenta < iMinPixels)
		{
			// Se decrementan los indices de blobs para quitar el actual
			for(yr=0; yr<wTamY; yr++)
				for(xr=0; xr<wTamX; xr++)
					// Borra blob actual
					if(pSalida[yr][xr] == b)
						pSalida[yr][xr]=0;
					// Decrementa indices del los blobs por encima del actual
					else if(pSalida[yr][xr] > b)
						pSalida[yr][xr]--;
			
			// Hay un blob menos
			bMax--;
			// Y hay que mirar de nuevo el actual
			b--;
		}
	}

	// Elimina blobs sobrantes
	for(yr=0; yr<wTamY; yr++)
		for(xr=0; xr<wTamX; xr++)
			if(pSalida[yr][xr] > bMax)
				pSalida[yr][xr]=0;			

	return (int)bMax;
}

// Calcula la centro de gravedad del eje principal del blob iBlob
void CentroGravedadXY(WORD wTamX, WORD wTamY, BYTE** pSalida, int iBlob, 
					  int& iCX, int& iCY)
{
	WORD y,x;
	int iCont_y=0;
	int iCont_x=0;
	int iCont_puntos=0;

	// Blob en byte
	BYTE bBlob = (BYTE)iBlob;

	// Suma de todos los puntos Y de un blob
	for(y=0; y<wTamY; y++)
		for(x=0; x<wTamX; x++)
			if(pSalida[y][x]==bBlob)
			{
				iCont_y += (int)y;
				iCont_x += (int)x;
				iCont_puntos++;
			}

	// Impide divisin por 0
	if (iCont_puntos == 0)
	{
		iCY = 0;
		iCX = 0;

		return;
	}
	
	// Dividido todos los puntos Y por el numero total de puntos
	iCY = iCont_y / iCont_puntos;
	iCX = iCont_x / iCont_puntos;
}

// Calcula el angulo del eje principal del blob iBlob (en grados)
float AnguloEje(WORD wTamX, WORD wTamY, BYTE** pSalida, int iBlob, int iCX, int iCY)
{
	WORD y,x;
	double dAux1, dAux2;

	// Blob en byte
	BYTE bBlob = (BYTE)iBlob;

	// Calculo de los momentos de orden pq para un blob determinado
	double dMomentos11=0.0;	// momento p=1 q=1
	double dMomentos20=0.0;	// momento p=2 q=0
	double dMomentos02=0.0;	// momento p=0 q=2

	for(y=0; y<wTamY; y++)
		for(x=0; x<wTamX; x++)
			if(pSalida[y][x]==bBlob)
			{
				dAux1 = (double)((int)x - iCX);
				dAux2 = (double)((int)y - iCY);

				dMomentos11 += pow(dAux1 ,1) * pow(dAux2 ,1);
				dMomentos20 += pow(dAux1 ,2) * pow(dAux2 ,0);
				dMomentos02 += pow(dAux1 ,0) * pow(dAux2 ,2);
			}

	// Calculo del angulo en radianes
	dAux1 = -0.5*atan2(2*dMomentos11, dMomentos20-dMomentos02);

	// Devuelve angulo en grado
	return (float)((180*dAux1)/3.14159);
}


// Escribir imagne con coordenadas de blobs
BOOL EscribeBlobs(CString& strSalida, int iNumBlobs, StBlob* pBlobs)
{
	CFile file;
	BOOL bOk=TRUE;
	
	// Cabecera de la imagen
	StCabecera cab;
	cab.wX = (WORD) 3;			// 3 floats por blob
	cab.wY = (WORD) iNumBlobs;	// iNumBlobs puede ser 0
	cab.wC = 1;
	cab.wB = sizeof(float);
	
	// Guarda salida
	if (file.Open(strSalida, CFile::modeCreate | CFile::modeWrite))
	{
		TRY
		{
			// Escribir cabecera
			file.Write(&cab, sizeof(cab));

			// Escribir numero de blobs: no hace falta, es el numero de filas
			// file.Write(&fNumBlobls, sizeof(float));
				
			// Escribir datos de blobs
			/*for (int i=0; i<iNumBlobs; i++)
				file.Write(pBlobs[i], sizeof(StBlob));*/

			// Escribir datos de blobs: Es mejor asi
			if (iNumBlobs>0)
				file.Write(pBlobs, iNumBlobs*sizeof(StBlob));
		}
		CATCH( CFileException, e )
		{
			bOk = FALSE;
		}
		END_CATCH
			
		file.Close();
	}
	
	return bOk;
}

// Escribir imagen etiquetada se salida
BOOL EscribeSalida(CString& strSalida, StCabecera* cab, BYTE** pImgSal)
{	
	CFile file;
	BOOL bOk=TRUE;
		
	// Guarda salida
	if (file.Open(strSalida, CFile::modeCreate | CFile::modeWrite))
	{
		TRY
		{
			// Escribir cabecera
			file.Write(cab, sizeof(StCabecera));
				
			// Escribir contenido por filas
			WORD wTamY = cab->wY;
			UINT uiTamX = (UINT)(cab->wX);
			for(WORD wY=0; wY<wTamY; wY++)
				file.Write(pImgSal[wY], uiTamX);
		}
		CATCH( CFileException, e )
		{
			bOk = FALSE;
		}
		END_CATCH
			
		file.Close();
	}
	else
		bOk = FALSE;

	return bOk;
}

// Funcion que calcula el momento de orden pq para un blob determinado
double Momento(int p, int q, int blob, BYTE **Matriz, 
			   WORD wTamX, WORD wTamY, StBlob* pBlobs)
{
	double sum=0;
	WORD x,y;
	double dAux1, dAux2;
  
	// Por si acaso: blobs vale 1, 2...
	if (blob<1)
		return sum;

	for(y=0; y<wTamY; y++)
		for(x=0; x<wTamX; x++)
			if(Matriz[y][x]==blob)
			{
				dAux1 = (double)((float)x - pBlobs[blob-1].fX);
				dAux2 = (double)((float)y - pBlobs[blob-1].fY);

				sum += pow(dAux1,p) * pow(dAux2,q);
			}
	      
	return sum;
}

// Funcion que calcula a partir de los momentos, los momentos centrales normalizados
// que son invariantes a posicion y escalado. Se le pasa como parametros la imagen, 
// el blob de la imagen del que se quiere calcular el momento y los ordenes p,q 
// de los momentos
double Momento_normalizado(int p, int q, int blob, BYTE **Matriz, 
						   WORD wTamX, WORD wTamY, StBlob* pBlobs)
{
	double momento_n=0.0;
	double n=0;
	double d=0;
	float k;
 
	// Error al calcular el momento normalizado, El orden p+q debe ser mayor que 2
	if ((p+q)<2)
	{
		AfxMessageBox("Error al calcular el momento normalizado:\nEl orden p+q debe ser mayor que 2");
		return 1000;
	}
	//Calculo de k
	else
	{
		k=((float)(p+q)/2)+1;
		//Llamada a momento de orden p,q, (Numerador)
		n=Momento(p, q, blob, Matriz, wTamX, wTamY, pBlobs);
		//Llamada a momento de orden 0,0 (Denominador)
		d=Momento(0, 0, blob, Matriz, wTamX, wTamY, pBlobs);
		//Momento normalizado
		momento_n=(double)n/(pow(d,k));
	}

  return momento_n;
}

// Calcula momento de Hu iOrden para el blob iBlob
float MomentoHu(WORD wTamX, WORD wTamY, int iBlob, int iOrden, BYTE** pSalida, 
				StBlob* pBlobs)
{
	int i = iBlob;
	double momento_hu=0.0;
	
	switch(iOrden)
	{
		case 1:		momento_hu=Momento_normalizado(2,0,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(0,2,i,pSalida, wTamX, wTamY, pBlobs);	
					break;

		case 2:		momento_hu=pow((Momento_normalizado(2,0,i,pSalida, wTamX, wTamY, pBlobs)-Momento_normalizado(0,2,i,pSalida, wTamX, wTamY, pBlobs)),2)+4*pow(Momento_normalizado(1,1,i,pSalida, wTamX, wTamY, pBlobs),2);	
					break;

		case 3:		momento_hu=pow((Momento_normalizado(3,0,i,pSalida, wTamX, wTamY, pBlobs)-3*Momento_normalizado(1,2,i,pSalida, wTamX, wTamY, pBlobs)),2)+pow((3*Momento_normalizado(2,1,i,pSalida, wTamX, wTamY, pBlobs)-Momento_normalizado(0,3,i,pSalida, wTamX, wTamY, pBlobs)),2);	
					break;

		case 4:		momento_hu=pow((Momento_normalizado(3,0,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(1,2,i,pSalida, wTamX, wTamY, pBlobs)),2)+pow((Momento_normalizado(2,1,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(0,3,i,pSalida, wTamX, wTamY, pBlobs)),2);			
					break;

		case 5:		momento_hu=(Momento_normalizado(3,0,i,pSalida, wTamX, wTamY, pBlobs)-3*Momento_normalizado(1,2,i,pSalida, wTamX, wTamY, pBlobs))*(Momento_normalizado(3,0,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(1,2,i,pSalida, wTamX, wTamY, pBlobs))*
					((pow((Momento_normalizado(3,0,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(1,2,i,pSalida, wTamX, wTamY, pBlobs)),2))-(3*pow((Momento_normalizado(2,1,i,pSalida, wTamX, wTamY, pBlobs)-Momento_normalizado(0,3,i,pSalida, wTamX, wTamY, pBlobs)),2)))+									
					(3*Momento_normalizado(2,1,i,pSalida, wTamX, wTamY, pBlobs)-Momento_normalizado(0,3,i,pSalida, wTamX, wTamY, pBlobs))*(Momento_normalizado(2,1,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(0,3,i,pSalida, wTamX, wTamY, pBlobs))*
					((3*pow((Momento_normalizado(3,0,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(1,2,i,pSalida, wTamX, wTamY, pBlobs)),2))-(pow((Momento_normalizado(2,1,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(0,3,i,pSalida, wTamX, wTamY, pBlobs)),2)));			
					break;

		case 6:		momento_hu=(Momento_normalizado(2,0,i,pSalida, wTamX, wTamY, pBlobs)-Momento_normalizado(0,2,i,pSalida, wTamX, wTamY, pBlobs))*
					((Momento_normalizado(3,0,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(1,2,i,pSalida, wTamX, wTamY, pBlobs))-(pow((Momento_normalizado(2,1,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(0,3,i,pSalida, wTamX, wTamY, pBlobs)),2)))+
					4*Momento_normalizado(1,1,i,pSalida, wTamX, wTamY, pBlobs)*(Momento_normalizado(3,0,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(1,2,i,pSalida, wTamX, wTamY, pBlobs))*
					(Momento_normalizado(2,1,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(0,3,i,pSalida, wTamX, wTamY, pBlobs));
					break;
					
		case 7:		momento_hu=(3*Momento_normalizado(2,1,i,pSalida, wTamX, wTamY, pBlobs)-Momento_normalizado(0,3,i,pSalida, wTamX, wTamY, pBlobs))*(Momento_normalizado(3,0,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(1,2,i,pSalida, wTamX, wTamY, pBlobs))*
					((pow((Momento_normalizado(3,0,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(1,2,i,pSalida, wTamX, wTamY, pBlobs)),2))-(3*pow((Momento_normalizado(2,1,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(0,3,i,pSalida, wTamX, wTamY, pBlobs)),2)))+
					(3*Momento_normalizado(2,1,i,pSalida, wTamX, wTamY, pBlobs)-Momento_normalizado(3,0,i,pSalida, wTamX, wTamY, pBlobs))*(Momento_normalizado(2,1,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(0,3,i,pSalida, wTamX, wTamY, pBlobs))*
					((3*pow((Momento_normalizado(3,0,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(1,2,i,pSalida, wTamX, wTamY, pBlobs)),2))-(pow((Momento_normalizado(2,1,i,pSalida, wTamX, wTamY, pBlobs)+Momento_normalizado(0,3,i,pSalida, wTamX, wTamY, pBlobs)),2)));
					break;
		
		//default:
		//			printf("El MOMENTO de HU debe ser de orden debe ser =>1 y <=7 NO->%d",orden);
		//			return 2000;
	}

	return (float)momento_hu;
}

// Escribe en un archivo de imagen float los valores de los momentos de Hu
BOOL EscribirMomentosHu(CString& strSalida, int iNumBlobs, float** pMomentos)
{
	CFile file;
	BOOL bOk=TRUE;
	
	// Cabecera de la imagen
	StCabecera cab;
	cab.wC = 1;
	cab.wB = sizeof(float);
	cab.wX = (WORD) 7;			// 7 floats por momento
	cab.wY = (WORD) iNumBlobs;	// Puede ser 0
	
	// Numero de blobs en float
	float fNumBlobls = (float)iNumBlobs;
		
	// Guarda salida
	if (file.Open(strSalida, CFile::modeCreate | CFile::modeWrite))
	{
		TRY
		{
			// Escribir cabecera
			file.Write(&cab, sizeof(cab));

			// Si hay blobs
			if (iNumBlobs>0)
			{
				// Escribir datos de momentos de Hu
				for (int i=0; i<iNumBlobs; i++)
					for (int j=0; j<7; j++)
						file.Write(&(pMomentos[j][i]), sizeof(float));
			}
		}
		CATCH( CFileException, e )
		{
			bOk = FALSE;
		}
		END_CATCH
			
		file.Close();
	}
	else
		bOk = FALSE;
	
	return bOk;
}